@@ -1323,6 +1323,16 @@ e_book_backend_exchange_create_contact (EBookBackendSync  *backend,
 		note = e_contact_get (*contact, E_CONTACT_NOTE);
 		photo = e_contact_get (*contact, E_CONTACT_PHOTO);
 
+		if (!bepriv->connected || !bepriv->ctx || !bepriv->summary) {
+			GNOME_Evolution_Addressbook_CallStatus state;
+
+			state = e_book_backend_exchange_connect (be);
+			if ( state != GNOME_Evolution_Addressbook_Success) {
+				d(printf("Returning status %d while creating contact\n", state));
+				return state;
+			}
+		}
+
 		status = e_folder_exchange_proppatch_new (bepriv->folder, NULL, name,
 							  test_name, bepriv->summary,
 							  props, &location, NULL);
@@ -1392,6 +1402,16 @@ e_book_backend_exchange_modify_contact (EBookBackendSync  *backend,
 		*contact = e_contact_new_from_vcard (vcard);
 		uri = e_contact_get_const (*contact, E_CONTACT_UID);
 
+		if (!bepriv->connected || !bepriv->ctx || !bepriv->summary) {
+			GNOME_Evolution_Addressbook_CallStatus state;
+
+			state = e_book_backend_exchange_connect (be);
+			if ( state != GNOME_Evolution_Addressbook_Success) {
+				d(printf("Returning status %d while modifying contact\n", state));
+				return state;
+			}
+		}
+
 		status = e2k_context_propfind (bepriv->ctx, NULL, uri,
 					       field_names, n_field_names,
 					       &results, &nresults);
@@ -2108,7 +2128,10 @@ e_book_backend_exchange_get_contact (EBookBackendSync  *backend,
 	EBookBackendExchange *be = E_BOOK_BACKEND_EXCHANGE (backend);
 	EBookBackendExchangePrivate *bepriv = be->priv;
 	EContact *contact;
-
+	E2kResult *results=NULL;
+	E2kHTTPStatus status;
+	int nresults = 0;
+	
 	d(printf("ebbe_get_contact(%p, %p, %s)\n", backend, book, id));
 
 	be = E_BOOK_BACKEND_EXCHANGE (e_data_book_get_backend (book));
@@ -2130,7 +2153,55 @@ e_book_backend_exchange_get_contact (EBookBackendSync  *backend,
 		}
 		
 	case GNOME_Evolution_Addressbook_MODE_REMOTE:
-		/* XXX finish this */
+
+		if (bepriv->marked_for_offline && e_book_backend_cache_is_populated (bepriv->cache)) {
+			contact = e_book_backend_cache_get_contact (bepriv->cache,
+								    id);
+			if (contact) {
+				*vcard =  e_vcard_to_string (E_VCARD (contact), 
+							     EVC_FORMAT_VCARD_30);
+				g_object_unref (contact);
+				return GNOME_Evolution_Addressbook_Success;
+			}
+			else {
+				*vcard = g_strdup ("");
+				return GNOME_Evolution_Addressbook_ContactNotFound;
+			}
+
+		} else {
+			E2kUri *uri;
+
+			/* Dont allow any non uri to go GAL UID is not a uri */
+			uri = e2k_uri_new (id);
+			if (!uri->protocol ||  !*uri->protocol) {
+				e2k_uri_free (uri);
+				*vcard = g_strdup ("");
+				return GNOME_Evolution_Addressbook_ContactNotFound;
+			}
+			e2k_uri_free (uri);
+
+			status = e2k_context_propfind (bepriv->ctx, NULL, id,
+						       field_names, n_field_names,
+						       &results, &nresults);
+		
+			if (status == E2K_HTTP_CANCELLED)
+				return GNOME_Evolution_Addressbook_OtherError;
+
+			if (status == E2K_HTTP_MULTI_STATUS && nresults > 0) {
+				contact = e_contact_from_props (be, &results[0]);
+				*vcard =  e_vcard_to_string (E_VCARD (contact), 
+							     EVC_FORMAT_VCARD_30);
+				g_object_unref (contact);
+				e2k_results_free (results, nresults);
+				
+				return GNOME_Evolution_Addressbook_Success;
+			
+			} else {
+				*vcard = g_strdup ("");
+				
+				return GNOME_Evolution_Addressbook_ContactNotFound;			
+			}
+		}
 
 	default:
 		break;
